/**
 * \file FIFO_queue.hh
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date February 2004
 * \brief The header file that defines a FIFO_queue object.
 *
 * A FIFO_queue is a specific type of Queue that implements the FIFO
 * replacement policy.
 **/



// ==================================================================
#if !defined (_FIFO_QUEUE_HH)
#define _FIFO_QUEUE_HH
// ==================================================================



// ==================================================================
// INCLUDES

#include "Queue.hh"
// ==================================================================



// ==================================================================
/**
 * \brief A FIFO_queue organizes and replaces pages by order of first
 *        use.
 **/
class FIFO_queue : public Queue {

public:

  /**
   * \brief The constructor.
   * \param memory_size The maximum number of pages managed by the
   *        FIFO queue.
   *
   * Prepare a FIFO queue by initializing the parent Queue
   * structures.
   **/
  FIFO_queue (const unsigned int memory_size);

protected: // METHODS

  /**
   * \brief Respond to a reference to a page already in the
   *        queue.
   * \param referenced_page_number The referenced page that caused the
   *        hit.
   * \param map_iterator An iterator into the map that connects the
   *        referenced page number to its list position.
   *
   * A page in the queue has been referenced.  Do nothing, since FIFO
   * requires no reordering of pages in the list.
   **/
  virtual void handle_hit (const page_number_t referenced_page_number,
			   page_map_t::iterator map_iterator);

  /** 
   * \brief Respond to a reference to a page that is not in the
   *        queue.
   * \param referenced_page_number The referenced page that caused the
   *        miss.
   *
   * A page that is not in the queue has been referenced.  If the
   * queue is full, evict the page that has been resident longest, and
   * insert the referenced page at the front of the list.
   **/
  virtual void handle_miss (const page_number_t referenced_page_number);

}; // class FIFO_queue
// ==================================================================



// ==================================================================
#endif // _FIFO_QUEUE_HH
// ==================================================================
