/**
 * \file LRU_queue.cc
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date February 2004
 * \brief The code file that defines a LRU_queue object.
 *
 * A LRU_queue is a specific type of Queue that implements the LRU
 * replacement policy.
 **/



// ==================================================================
// INCLUDES

#include <assert.h>
#include "LRU_queue.hh"
// ==================================================================



// ==================================================================
LRU_queue::LRU_queue (const unsigned int memory_size) :
  Queue(memory_size) {

  // Nothing else to do here.

} // Queue::Queue
// ==================================================================



// ==================================================================
void
LRU_queue::handle_hit (const page_number_t referenced_page_number,
		       page_map_t::iterator map_iterator) {

  // Erase the page from its current location in the list...
  page_list.erase((*map_iterator).second);

  // ...and insert it at the head of the list.
  page_list.push_front(referenced_page_number);
  (*map_iterator).second = page_list.begin();

} // LRU_queue::handle_hit
// ==================================================================



// ==================================================================
void
LRU_queue::handle_miss (const page_number_t referenced_page_number) {

  // Insert the new page.
  page_list.push_front(referenced_page_number);
  page_map[referenced_page_number] = page_list.begin();
  pages_held++;

  // Is the queue now overly-full?
  if (pages_held > memory_size) {

    // Yes.  Evict the least recently used page.
    page_number_t evicted_page = page_list.back();
    page_list.pop_back();
    page_map_t::iterator i = page_map.find(evicted_page);
    assert(i != page_map.end());
    page_map.erase(i);
    pages_held--;
    assert(pages_held == memory_size);

  }
} // LRU_queue::handle_miss
// ==================================================================
