/**
 * \file Queue.hh
 * \author Scott F. H. Kaplan <sfkaplan@cs.amherst.edu>
 * \date February 2004
 * \brief The code file that defines the abstract class Queue.
 *
 * A Queue organizes the pages of a fixed-size memory, into a list,
 * and provides a hash map for constant-time lookup into that list.
 * It implements no replacement policy, and so it is only a abstract
 * class, where subclasses must handle replacement and list
 * re-ordering.
 **/



// ==================================================================
// INCLUDES

#include "Queue.hh"
// ==================================================================



// ==================================================================
Queue::Queue (const unsigned int memory_size) :
  memory_size(memory_size),
  pages_held(0) {

  // Nothing else to do here.

} // Queue::Queue
// ==================================================================



// ==================================================================
bool
Queue::touch (const page_number_t referenced_page_number) {

  // Attempt to lookup the reference page through the map.
  page_map_t::iterator map_iterator =
    page_map.find(referenced_page_number);

  // Was the page found?
  if (map_iterator == page_map.end()) {

    // No.  Handle the miss and return that the page was not resident.
    handle_miss(referenced_page_number);
    return false;

  } else {

    // Yes.  Handle the hit and return that the page was resident.
    handle_hit(referenced_page_number, map_iterator);
    return true;

  }
} // Queue::touch
// ==================================================================
